/*
 * #%L
 * elk-reasoner
 * 
 * $Id: ElkSubObjectPropertyOfAxiom.java 71 2011-06-05 11:11:41Z mak@aifb.uni-karlsruhe.de $
 * $HeadURL: http://elk-reasoner.googlecode.com/svn/tags/elk-parent-0.1.0/elk-reasoner/src/main/java/org/semanticweb/elk/syntax/ElkSubObjectPropertyOfAxiom.java $
 * %%
 * Copyright (C) 2011 Oxford University Computing Laboratory
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
/**
 * @author Yevgeny Kazakov, Apr 8, 2011
 */
package org.semanticweb.elk.syntax;

import org.semanticweb.elk.util.HashGenerator;

/**
 * Corresponds to an <a href=
 * "http://www.w3.org/TR/owl2-syntax/#Object_Subproperties">Object Subproperty
 * Axiom<a> in the OWL 2 specification.
 * 
 * @author Yevgeny Kazakov
 * 
 */
public class ElkSubObjectPropertyOfAxiom extends ElkObjectPropertyAxiom {
	protected final ElkObjectPropertyExpression subObjectPropertyExpression,
			superObjectPropertyExpression;

	private static final int constructorHash_ = "ElkSubObjectPropertyOfAxiom"
			.hashCode();

	private ElkSubObjectPropertyOfAxiom(
			ElkObjectPropertyExpression subObjectPropertyExpression,
			ElkObjectPropertyExpression superObjectPropertyExpression) {
		this.subObjectPropertyExpression = subObjectPropertyExpression;
		this.superObjectPropertyExpression = superObjectPropertyExpression;
		this.structuralHashCode = HashGenerator.combineListHash(
				constructorHash_, subObjectPropertyExpression
						.structuralHashCode(), superObjectPropertyExpression
						.structuralHashCode());
	}

	public ElkObjectPropertyExpression getSubObjectPropertyExpression() {
		return subObjectPropertyExpression;
	}

	public ElkObjectPropertyExpression getSuperObjectPropertyExpression() {
		return superObjectPropertyExpression;
	}

	public static ElkSubObjectPropertyOfAxiom create(
			ElkObjectPropertyExpression subObjectPropertyExpression,
			ElkObjectPropertyExpression superObjectPropertyExpression) {
		return (ElkSubObjectPropertyOfAxiom) factory
				.put(new ElkSubObjectPropertyOfAxiom(
						subObjectPropertyExpression,
						superObjectPropertyExpression));
	}

	@Override
	public String toString() {
		StringBuilder result = new StringBuilder("SubObjectPropertyOf(");
		result.append(subObjectPropertyExpression.toString());
		result.append(" ");
		result.append(superObjectPropertyExpression.toString());
		result.append(")");
		return result.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.semanticweb.elk.reasoner.ElkObject#structuralEquals(java.lang.Object)
	 */
	@Override
	public boolean structuralEquals(ElkObject object) {
		if (this == object)
			return true;

		if (object instanceof ElkSubObjectPropertyOfAxiom)
			return subObjectPropertyExpression
					.equals(((ElkSubObjectPropertyOfAxiom) object).subObjectPropertyExpression)
					&& superObjectPropertyExpression
							.equals(((ElkSubObjectPropertyOfAxiom) object).superObjectPropertyExpression);

		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.semanticweb.elk.reasoner.ELKObjectPropertyAxiom#accept(org.semanticweb
	 * .elk.reasoner.ELKObjectPropertyAxiomVisitor)
	 */
	@Override
	public <O> O accept(ElkObjectPropertyAxiomVisitor<O> visitor) {
		return visitor.visit(this);
	}

}
